import 'package:chhota_stocks_flutter/l10n/translations.dart';
import 'package:chhota_stocks_flutter/src/core/core.dart';
import 'package:chhota_stocks_flutter/src/feature/commonWidget/common_widgets.dart';
import 'package:chhota_stocks_flutter/src/feature/initSDK/init_sdk.dart';
import 'package:chhota_stocks_flutter/src/feature/orderStatement/ui/order_statement_widget.dart';
import 'package:chhota_stocks_flutter/src/feature/transactionCallback/bloc/transaction_callback_cubit.dart';
import 'package:chhota_stocks_flutter/src/feature/transactionCallback/model/transaction_callback_onsuccess.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'dart:convert';

class TransactionLoader extends StatelessWidget {
  final String? data;
  const TransactionLoader({Key? key, this.data}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    return BlocProvider<TransactionCallbackCubit>(
      create: (_) => TransactionCallbackCubit(),
      child: _TxnLoader(
        data: data,
      ),
    );
  }
}

class _TxnLoader extends StatefulWidget {
  final String? data;
  const _TxnLoader({Key? key, this.data}) : super(key: key);

  @override
  State<_TxnLoader> createState() => _TxnLoaderState();
}

class _TxnLoaderState extends State<_TxnLoader> {
  @override
  void initState() {
    super.initState();

    submitTransactionId();
  }

  void submitTransactionId() {
    TransactionCallbackOnSuccess? transactionCallbackOnSuccess;

    try {
      if (widget.data != null && widget.data!.isNotEmpty) {
        final value = jsonDecode(widget.data!);

        if(value["errorCode"]!=null){
          transactionCallbackOnSuccess =
              TransactionCallbackOnSuccess.fromJson(value);
        }else{
          transactionCallbackOnSuccess =
              TransactionCallbackOnSuccess.fromJson(value["data"]);
        }
      }
    } catch (e) {
      debugPrint("Error on txn callback::${e.toString()}");
    }
    if (transactionCallbackOnSuccess?.success != null &&
        transactionCallbackOnSuccess!.success == false &&
        transactionCallbackOnSuccess.error != null &&
        transactionCallbackOnSuccess.error!.contains("market_closed")) {

      BlocProvider.of<TransactionCallbackCubit>(context).marketClosed(transactionCallbackOnSuccess.errorCode);

    } else if (transactionCallbackOnSuccess?.smallcaseAuthToken == null ||
        transactionCallbackOnSuccess?.transactionId == null) {
      Navigator.pop(context);
    } else if (transactionCallbackOnSuccess?.smallcaseAuthToken != null &&
        transactionCallbackOnSuccess?.transactionId != null) {
      BlocProvider.of<TransactionCallbackCubit>(context).transactionCallback(
        txnId: transactionCallbackOnSuccess!.transactionId!,
        smallcaseAuthId: transactionCallbackOnSuccess.smallcaseAuthToken!,
      );
    } else {
      BlocProvider.of<TransactionCallbackCubit>(context).txnFailed();
    }
  }

  @override
  Widget build(BuildContext context) {
    return WillPopScope(
        child: Theme(
          data: ChhotaStocksTheme.lightThemeData,
          child: Material(
            child:
                BlocBuilder<TransactionCallbackCubit, TransactionCallbackState>(
              builder: (context, state) {
                if(state is MarketClosedTransaction){
                    return MarketClosedTxt(onDone: ()=>Navigator.pop(context),errorCode: state.errorCode,);

                }else if (state is TransactionCallbackError) {
                  return ChhotaStocksErrorWidget(
                    onPressed: () {
                      if (widget.data == null) {
                        Navigator.pop(context);
                      } else {
                        submitTransactionId();
                      }
                    },
                  );
                }

                return AnimatedSwitcher(
                  duration: const Duration(milliseconds: 500),
                  transitionBuilder:
                      (Widget child, Animation<double> animation) {
                    return FadeTransition(opacity: animation, child: child);
                  },
                  child: state is TransactionCallbackSuccess
                      ? OrderSubmittedTxt(
                          onDone: () => Navigator.pushAndRemoveUntil(
                              context,
                              MaterialPageRoute(
                                  builder: (context) => OrderStatmentWidget(
                                        onBack: () {
                                          Navigator.pushAndRemoveUntil(
                                              context,
                                              MaterialPageRoute(
                                                  builder: (context) =>
                                                      const InitSDKWidget()),
                                              (route) => false);
                                        },
                                      )),
                              (route) => false),
                        )
                      : const Loader(),
                );
              },
            ),
          ),
        ),
        onWillPop: () async {
          return false;
        });
  }
}

class OrderSubmittedTxt extends StatelessWidget {
  final VoidCallback onDone;
  const OrderSubmittedTxt({Key? key, required this.onDone}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: Alignment.center,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          const CircleAvatar(
              radius: 80,
              backgroundColor: Colors.green,
              child: Icon(
                Icons.done_outline_outlined,
                color: Colors.white,
                size: 50,
              )),
          const SizedBox(
            height: 20,
          ),
          Text(
            Translations.of(context).orderSubmitted,
            overflow: TextOverflow.ellipsis,
          ),
          const SizedBox(
            height: 32,
          ),
          ElevatedButton(
              onPressed: onDone,
              child: Padding(
                padding: EdgeInsets.symmetric(horizontal: 30.0, vertical: 10.0),
                child: Text(
                  Translations.of(context).viewOrder,
                  //   "View Orders",
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(fontSize: 16, color: Colors.white),
                ),
              )),
        ],
      ),
    );
  }
}


class MarketClosedTxt extends StatelessWidget {
  final int? errorCode;
  final VoidCallback onDone;
  const MarketClosedTxt({Key? key, required this.onDone,this.errorCode}) : super(key: key);
  @override
  Widget build(BuildContext context) {
    return Container(
      alignment: Alignment.center,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          const CircleAvatar(
              radius: 60,
              backgroundColor: Colors.white,
              child: Icon(
                Icons.error_outline,
                color: Color(0xffff5722),
                size: 120,
              )),
          const SizedBox(
            height: 20,
          ),

          if(errorCode!=null&&errorCode==4005)
           const  Padding(
              padding:  EdgeInsets.symmetric(horizontal: 20.0),
              child:  Text(
                "The transaction could not processed as After Market Order(AMO) orders because either of these:\n\n1. broker does not support AMO orders\n\n2. particular chhotastock order type cannot be executed in after-market hours (ex - rebalance)"
              ),
            )
          else
            const Text(
            "Market is closed.Please try later.",
            overflow: TextOverflow.ellipsis,
            style: TextStyle(
              fontSize: 20
            ),
          ),

          const SizedBox(height: 16.0,),

          const Padding(
            padding:  EdgeInsets.symmetric(horizontal: 20.0),
            child:  Text(
              "* Market hours are from 9:15 a.m to 3:25 p.m any orders placed after this will be place as an AMO order",
              style: TextStyle(
                color: Color(0xffff5722),
                fontSize: 12.0
              ),
            ),
          ),

          const SizedBox(
            height: 32,
          ),
          ElevatedButton(
              onPressed: onDone,
              child: const Padding(
                padding: EdgeInsets.symmetric(horizontal: 30.0, vertical: 10.0),
                child:  Text(
                  "Back",
                  overflow: TextOverflow.ellipsis,
                  style: TextStyle(fontSize: 16, color: Colors.white),
                ),
              )),
        ],
      ),
    );
  }
}
